#include "gdrom.h"
#include "proto.h"

#ifndef NULL
#define NULL ((void*)0)
#endif


/*
 * GDROM system access
 */

int exec_cmd(int cmd, void *param)
{
  int status[4];
  int f, n;

  /* Submit the command to the system */
  f = gdGdcReqCmd(cmd, param);

  /* Wait for the command to complete */
  do gdGdcExecServer(); while((n = gdGdcGetCmdStat(f, status))==1);

  /* If command completed normally, return 0 */
  if(n == 2)
    return 0;

  /* Detect conditions that have special error codes */
  else switch(status[0]) {
   case 2:
     return ERR_NODISK;
   case 6:
     return ERR_DISKCHG;
   default:
     return ERR_SYSERR;
  }
}

int gdrom_init_drive()
{
  int i, r;
  unsigned int param[4];
  int cdxa;

  /*
   * Retry the init disc command a few times if it fails,
   * for example if the disc has been changed
   */
  for(i=0; i<8; i++)
    if(!(r = exec_cmd(24, NULL)))
      break;

  /* If it failed all times, give up */
  if(r)
    return r;

  /* Check type of disc */
  gdGdcGetDrvStat(param);

  /* CD/XA? */
  cdxa = (param[1] == 32);

  /* Select disc access mode */
  param[0] = 0;                   /* 0 = set, 1 = get */
  param[1] = 8192;                /* ?                */
  param[2] = (cdxa? 2048 : 1024); /* mode 1/2         */
  param[3] = 2048;                /* sector size      */
  if(gdGdcChangeDataType(param)<0)
    return ERR_SYSERR;

  /* All done */
  return 0;
}

int gdrom_get_status()
{
  unsigned int param[2];

  gdGdcGetDrvStat(param);

  return param[0];
}


int gdrom_read_toc(TOC *toc, int session)
{
  struct { int session; void *buffer; } param;
  param.session = session;
  param.buffer = toc;
  return exec_cmd(19, &param);
}

int gdrom_read_sectors(char *buf, int sec, int num)
{
  struct { int sec, num; void *buffer; int dunno; } param;
  int i;
  param.sec = sec;    /* Starting sector number */
  param.num = num;    /* Number of sectors      */
  param.buffer = buf; /* Pointer to buffer      */
  param.dunno = 0;
  return exec_cmd(16, &param);
}

void gdrom_init()
{
  register unsigned long p, x;

  /* Reactivate GD-ROM drive */

  *((volatile unsigned long *)0xa05f74e4) = 0x1fffff;
  for(p=0; p<0x200000/4; p++)
    x = ((volatile unsigned long *)0xa0000000)[p];

  /* Reset GD system functions */

  gdGdcInitSystem();
}

/*******************---------------------------------------******************/

unsigned int last_sect;

int cd_start()
{
	int i,lt,lo;
	TOC toc;

	lcd_puts(0,"starting CD");
	do {
		gdrom_init();
		gdrom_init_drive();
		i=gdrom_read_toc(&toc,0);
	} while (i<0);
	lt=TOC_TRACK(toc.last)-1;
	i=TOC_LBA(toc.entry[lt]);
	last_sect=TOC_LBA(toc.leadout);
	
	return i-150;
}
